<?php
/**
 * @category    Klaviyo
 * @package     Klaviyo_Reclaim
 * @copyright   Copyright (c) 2013 Klaviyo Inc. (http://www.klaviyo.com)
 */


/**
 * Klaviyo Api
 *
 * @category   Klaviyo
 * @package    Klaviyo_Reclaim
 * @author     Klaviyo Team <support@klaviyo.com>
 */

use Klaviyo_Reclaim_Block_Adminhtml_System_Config_Fieldset_Info;
class Klaviyo_Reclaim_Model_KlaviyoApi
{
    const LISTS_MEMBERS_ENDPOINT = 'lists/%s/relationships/profiles';
    const API_LISTS = 'lists';
    const API_LIST = 'list/';
    const HTTP_POST = 'POST';
    const HTTP_GET = 'GET';
    const HTTP_DELETE = 'DELETE';
    const EMAIL_KEY = 'email';
    const EMAILS_KEY = 'emails';
    const PROFILES_KEY = 'profiles';
    const DATA_KEY = 'data';
    const TYPE_KEY = 'type';
    const ID_KEY = 'id';
    const ATTRIBUTES_KEY = 'attributes';
    const PROFILE_SUBSCRIPTION_BULK_CREATE_JOBS = 'profile-subscription-bulk-create-job';
    const PROFILE_SUBSCRIPTION_BULK_DELETE_JOBS = 'profile-subscription-bulk-delete-job';
    const LIST_KEY = 'list';
    const RELATIONSHIPS_KEY = 'relationships';
    const SUBSCRIPTIONS_KEY = 'subscriptions';
    const PROFILE_KEY = 'profile';

    var $api_revision = '2023-08-15';
    var $api_url;

    var $error_message;
    var $error_code;

    /**
     * Default to a 300 second timeout on server calls
     */
    var $timeout = 300;

    /**
     * Default to a 8K chunk size
     */
    var $chunk_size = 8192;

    /**
     * Cache the user api_key so we only have to log in once per client instantiation
     */
    var $api_key;

    /**
     * @var array Request params storage
     */
    public $request_params = array();

    /**
     * Connect to the Klaviyo API for a given list.
     *
     * @param string $apikey Your Klaviyo apikey
     * @param string $secure Whether or not this should use a secure connection
     */
    function __construct($api_key, $secure=true) {
        $this->api_key = $api_key;
        $this->secure = $secure;
        $this->api_base_url = 'https://a.klaviyo.com/api/';
    }

    function setTimeout($seconds) {
        if (is_int($seconds)) {
            $this->timeout = $seconds;
            return true;
        }
    }

    function getTimeout() {
        return $this->timeout;
    }

    function lists() {
        return $this->callServer(self::HTTP_GET, self::API_LISTS);
    }

    function searchProfileByEmail($email) {
        $url_path_query = "profiles/?filter=equals(email,'$email')";
        $response = $this->callServer(self::HTTP_GET, $url_path_query);

        if (empty($response['data']) || !$response) {
            return false;
        } else {
            return $response[0]['id'];
        }
    }

    function listMembersAdd($list_id, $profile_id) {
        $body = $this->listMemberRequestBody($profile_id);
        return $this->callServer(self::HTTP_POST, sprintf(self::LISTS_MEMBERS_ENDPOINT, $list_id), $body);
    }

    function listMembersDelete($list_id, $profile_id) {
        $body = $this->listMemberRequestBody($profile_id);
        return $this->callServer(self::HTTP_DELETE, sprintf(self::LISTS_MEMBERS_ENDPOINT, $list_id), $body);
    }

    function listSubscriberAdd($list_id, $email) {
        $body = $this->listSubscriptionRequestBody($email, $list_id, true);
        return $this->callServer(self::HTTP_POST, self::PROFILE_SUBSCRIPTION_BULK_CREATE_JOBS, $body);
    }

    function listSubscriberDelete($list_id, $email) {
        $body = $this->listSubscriptionRequestBody($email, $list_id, false);
        return $this->callServer(self::HTTP_DELETE, self::PROFILE_SUBSCRIPTION_BULK_DELETE_JOBS, $body);
    }

    function listMemberRequestBody($profile_id) {
        return array(
            self::DATA_KEY => array(
                array(
                    self::TYPE_KEY => 'profile',
                    self::ID_KEY => $profile_id
                )
            )
        );
    }

    function listSubscriptionRequestBody($email, $list_id, $create) {
        return array(
            self::DATA_KEY => array(
                self::TYPE_KEY => $create ? self::PROFILE_SUBSCRIPTION_BULK_CREATE_JOBS : self::PROFILE_SUBSCRIPTION_BULK_DELETE_JOBS,
                self::ATTRIBUTES_KEY => array(
                    'custom_source' => 'Magento',
                    self::PROFILES_KEY => array(
                        self::DATA_KEY => array(
                            array(
                                self::TYPE_KEY => self::PROFILE_KEY,
                                self::ATTRIBUTES_KEY => array(
                                    self::EMAIL_KEY => $email,
                                    self::SUBSCRIPTIONS_KEY => array(
                                        self::EMAIL_KEY => array(
                                            'MARKETING'
                                        )
                                    )
                                )
                            )
                        )
                    ),
                ),
                self::RELATIONSHIPS_KEY => array(
                    self::LIST_KEY => array(
                        self::DATA_KEY => array(
                            self::TYPE_KEY => self::LIST_KEY,
                            self::ID_KEY => $list_id
                        )
                    )
                )
            ),
        );
    }

    function callServer($method, $path, $body = null) {

        $this->errorMessage = '';
        $this->errorCode = '';

        $client = new Zend_Http_Client($this->api_base_url . $path);
        $client->setMethod($method);
    
        $client->setHeaders(array(
            "Content-Type" => "application/json",
            "Accept" => "application/json",
            "revision" => $this->api_revision,
            "Authorization" => "Klaviyo-API-Key $this->api_key",
            "X-Klaviyo-User-Agent" => 'magento-klaviyo/' . Klaviyo_Reclaim_Block_Adminhtml_System_Config_Fieldset_Info::getKlaviyoVersion() . ' PHP/' . phpversion()
        ));

        if (isset($body)) {
            $client->setParameterPost(json_encode($body));
        }

        try {
            $response = $client->request();
            if (!$response->isSuccessful()) {
                $this->error_message = $response->getBody();
                $this->error_code = $response->getStatus();
                return false;
            }
        } catch (Exception $ex) {
            Mage::logException($ex);
            return $ex->getMessage();
        }

        $json_response = Zend_Json::decode($response->getBody());

        if(is_array($json_response) && isset($json_response['errors'])) {
            $this->error_message = $json_response[0]['detail'];
            $this->error_code = $response->getStatus();
            return false;

        }

        return $json_response;
    }

}